# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from xml.sax import ContentHandler, parse

from CommandStore import *

class CommandsParser(ContentHandler):
	"""
	This parses the commands.xml
	"""
	def __init__(self):
		self.__commandstore = None
		
		self.__command = None
		self.__variant = None
		
	def startElement(self, name, attributes):
		if name == "cmd":
			try:
				package = attributes["package"]
			except KeyError:
				package = None
			
			self.__command = Command(attributes["name"], package)
			self.__variant = None
			
			self.__commandstore[self.__command.name] = self.__command
			
		elif name == "var":
			self.__variant = Variant(self.__command)		# we need a back-reference here
			self.__command.variants.append(self.__variant)
		
		elif name == "arg":
			try:
				if attributes["type"] == "optional":
					self.__argument = OptionalArgument(attributes["label"])
				elif attributes["type"] == "vector":
					self.__argument = VectorArgument()
				else:
					assert False
			except KeyError:
				self.__argument = MandatoryArgument(attributes["label"])
			
			if self.__variant:
				self.__variant.arguments.append(self.__argument)
			else:
				# automatically create a variant
				self.__variant = Variant(self.__command)	# we need a back-reference here
				self.__variant.arguments.append(self.__argument)
				self.__command.variants.append(self.__variant)
			
		elif name == "cho":
			try:
				p = attributes["package"]
			except KeyError:
				p = None
			
			# we need to call append_choice() here because [Mandatory|Optional]Argument.choices 
			# evaluates placeholders 
			self.__argument.append_choice(Choice(attributes["name"], p))
		
		elif name == "cmp":
			self.__argument.components.append(Component(attributes["label"]))
		
		elif name == "placeholder":
			# TODO: allow placeholders for more nodes than only Choices
			p = Placeholder()
			self.__argument.children.append(p)
			self.__commandstore.register_placeholder(attributes["key"], p)
			
			
	def parse(self, commandstore, filename):
		"""
		Parse XML
		"""
		self.__commandstore = commandstore
		parse(filename, self)


if __name__ == "__main__":
	from time import time
	
	t0 = time()
	
	s = CommandStore()
	p = CommandsParser()
	p.parse(s, "commands.xml")
	
	dt = "%.2f" % (time() - t0)
	
	print "SAX : " + dt
	
	for command in s.itervalues():
		print command.name
		for var in command.variants:
			for arg in var.arguments:
				print arg
	
	