# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from BibTeXFile import BibTeXFile
from LaTeXPlugin.Logger import error, debug

class BibTeXModelFactory(object):
	"""
	This produces BibTeXModel instances and is therefor able to cache them.
	
	(not BibTeXOutlineModels because they are generated from the current Document content 
	when a BibTeX file is edited)
	"""
	def __init__(self):
		self.__files = {}
	
	def get_model(self, filename):
		"""
		Return a model for the given filename (.bib)
		"""
		try:
			f = self.__files[filename]
			debug(self, "get_model", "Cache hit for %s" % filename)
			if f.changed:
				f.synchronize()
			return f.model
		except KeyError:
			debug(self, "get_model", "Cache miss for %s" % filename)
			try:
				f = BibTeXFile(filename)
				self.__files[filename] = f
				return f.model
			except Exception, e:
				error(self, "get_model", "Could not get model for %s: %s" % (filename, e))
		