# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from os.path import splitext
from re import compile, MULTILINE

from Message import *
from Processor import Processor
from LaTeXPlugin.Logger import debug

class LatexProcessor(Processor):
	"""
	This processor generates messages from a standard LaTeX log with
	default error format (NOT using "-file-line-error")
	"""
	def __init__(self):
		Processor.__init__(self)
		
		# matches error messages in LaTeX log file
		self.__pattern = compile(r"(^! (?P<text>.*?)$)|(^l\.(?P<line>[0-9]+))", MULTILINE)
	
	def get_messages(self):
		try:
			f = "%s.log" % splitext(self.filename)[0]
			s = open(f).read()
			
			if self.__file_line_error_enabled(s):
				return [Message(None, None, "The file:line:error format is not supported. Please remove that switch.")]
			
			messages = []
			
			for tuple in self.__process_log(s):
				line = int(tuple[1]) - 1
				messages.append(Message(self.filename, line, None, tuple[0], ERROR))
				
			return messages
		except IOError:
			return [Message(self.filename, None, "LaTeX log file not found", ERROR)]
	
	def __process_log(self, string):
		tuples = []
		for match in self.__pattern.finditer(string):
			if match.group("text"):
				tuple = [match.group("text"), 0]
			elif match.group("line"):
				tuple[1] = match.group("line")
				tuples.append(tuple)
		
		return tuples
	
	def __file_line_error_enabled(self, string):
		return bool(string.find("file:line:error") > 0)
