# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

LABEL, BIBITEM = 1, 2

## environments
UNKNOWN, TEXT, MATH = 0, 1, 2

## image formats
PNG, JPG, GIF = 1, 2, 4

def escape_text(s):
	"""
	Escapes text so that the markup parser will parse it verbatim. Less than, 
	greater than, ampersand, etc. are replaced with the corresponding entities.
	"""
	return s.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")

class ConditionalSource(object):
	"""
	This encapsulates LaTeX source that requires certain packages
	"""
	def __init__(self, source, packages):
		self.source, self.packages = source, packages
	
	def get_source(self):
		return self.source
	
	def get_required_packages(self):
		return self.packages

class PrefaceSource(object):
	"""
	This encapsulates a source that is to be inserted into the document's
	preface section 
	
	This type of source may only contain a string 
	"""
	def __init__(self, source):
		self.source = str(source)
	
	def get_source(self):
		return self.source
