# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from LaTeXPlugin.outline import OutlineView
from LaTeXPlugin.Logger import debug

from LaTeXPlugin.bibtex.StringReader import StringReader
from LaTeXPlugin.bibtex.BibTeXParser import BibTeXParser
from LaTeXPlugin.bibtex.BibTeXOutlineModel import BibTeXOutlineModel


class BibtexLanguageHelper(object):
	def __init__(self, document_helper):
		self.__document_helper = document_helper
		self.__view = document_helper.view
		
		self.__parser = BibTeXParser()
		
		self.__outline_model = BibTeXOutlineModel()
		
		self.__outline_view = OutlineView(self.__outline_model)
		self.__outline_view.connect("node-selected", self.__outline_node_selected)
		
		self.__document_helper.connect("document-changed", self.__document_changed)
	
	def __outline_node_selected(self, outline_view, line, column):
		debug(self, "__outline_node_selected")
		
		self.__document_helper.jump_to_line(line)
	
	def __document_changed(self, document_helper):
		self.__outline_model.clear()
		self.__parser.parse(self.__outline_model, StringReader(self.__document_helper.text))
	
	def insert_source(self, s):
		self.__buffer = self.__document_helper.view.get_buffer()
		self.__buffer.insert_at_cursor(s)
		
	@property
	def outline_view(self):
		return self.__outline_view
	