# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import ScrolledWindow, POLICY_AUTOMATIC, SHADOW_NONE, TreeView, \
		CellRendererPixbuf, CellRendererText, TreeViewColumn, VBox
from gobject import SIGNAL_RUN_LAST, TYPE_NONE, TYPE_INT, TYPE_STRING

from Logger import debug


class OutlineNodePosition(object):
	"""
	"""
	def __init__(self, line, column):
		self.__line = line
		self.__column = column
	
	@property
	def line(self):
		return self.__line
	
	@property
	def column(self):
		return self.__column


class OutlineNodeRange(object):
	"""
	"""
	def __init__(self, start, end):
		self.__start = start
		self.__end = end
	
	@property
	def start(self):
		return self.__start
	
	@property
	def end(self):
		return self.__end
	

class FileReference(OutlineNodeRange):
	"""
	"""
	def __init__(self, start, end, filename):
		OutlineNodeRange.__init__(self, start, end)
		self.__filename = filename
	
	@property
	def filename(self):
		return self.__filename


class OutlineViewContainer(VBox):
	"""
	This is a simple container for the OutlineView
	"""
	def __init__(self):
		VBox.__init__(self)
		self.__outline_view = None
		self.show_all()
	
	def set_outline_view(self, view):
		"""
		Display a new OutlineView in this container
		"""
		if self.__outline_view:
			self.remove(self.__outline_view)
		
		self.__outline_view = view
		
		if view:
			self.add(view)


class OutlineView(ScrolledWindow):
	
	__gsignals__ = {
		"node-selected" : (SIGNAL_RUN_LAST, TYPE_NONE, ([TYPE_INT, TYPE_INT])),			# line, column
		"node-range-selected" : (SIGNAL_RUN_LAST, TYPE_NONE, ([TYPE_INT, TYPE_INT])),	# start, end
		"file-activated" : (SIGNAL_RUN_LAST, TYPE_NONE, ([TYPE_STRING]))				# filename
	}
	
	def __init__(self, treestore=None):
		ScrolledWindow.__init__(self)
		self.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		self.set_shadow_type(SHADOW_NONE)
		
		self.__treestore = treestore
		
		self.__treeview = TreeView(treestore)
		self.__treeview.set_headers_visible(False)
		
		column = TreeViewColumn()
		
		pixbuf_renderer = CellRendererPixbuf()
		column.pack_start(pixbuf_renderer, False)
		column.add_attribute(pixbuf_renderer, "pixbuf", 1)
		
		text_renderer = CellRendererText()
		column.pack_start(text_renderer, True)
		column.add_attribute(text_renderer, "markup", 0)
		
		self.__treeview.append_column(column)
		
		self.__treeview.connect("cursor-changed", self.__cursor_changed)
		self.__treeview.connect("row-activated", self.__row_activated)
		
		self.add(self.__treeview)
		
		self.show_all()
	
	def __cursor_changed(self, treeview):
		store, it = treeview.get_selection().get_selected()
		if not it: 
			return
			
		node = store.get_value(it, 2)
		
		if isinstance(node, OutlineNodePosition):
			self.emit("node-selected", node.line, node.column)
		elif isinstance(node, OutlineNodeRange):
			self.emit("node-range-selected", node.start, node.end)
	
	def __row_activated(self, treeview, path, column):
		debug(self, "__row_activated")
		
		it = self.__treestore.get_iter(path)
		node = self.__treestore.get(it, 2)[0]
		
		if type(node) is FileReference:
			self.emit("file-activated", node.filename)
	
	def expand_all(self):
		self.__treeview.expand_all()
		