# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import Image, RESPONSE_OK

from LaTeXPlugin.Installation import Installation
from LaTeXPlugin.Logger import debug
from SnippetDialog import SnippetDialog, ACTION_NEW, ACTION_EDIT
from SnippetView import SnippetView
from Snippet import Snippet
from LaTeXPlugin.document.DocumentHelper import LANGUAGE_LATEX

class SnippetController(object):
	"""
	Controller for snippets
	"""
	def __init__(self, window_helper):
		self.__window_helper = window_helper
		self.__settings = window_helper.settings
		self.__dialog = None
		
		debug(self, "__init__")
		
		self.__view = SnippetView()
		
		self.__view.connect("snippet-activated", self.__snippet_activated)
		self.__view.connect("new-clicked", lambda x: self.__add_snippet())
		self.__view.connect("remove-clicked", self.__remove_snippet)
		self.__view.connect("edit-clicked", self.__edit_snippet)
		self.__view.connect("drag-text-received", self.__drag_text_received)
		
		image = Image()
		image.set_from_file(Installation().path + "/pixmaps/snippet.png")
		
		self.__window_helper.window.get_side_panel().add_item(self.__view, "Snippets", image)
		
		self.__update_snippets()
		
		debug(self, "__init__", "done")
	
	def deactivate(self):
		self.__window_helper.window.get_side_panel().remove_item(self.__view)
	
	def __add_snippet(self, source=""):
		if not self.__dialog:
			self.__dialog = SnippetDialog()
		
		snippet = Snippet(source=source)
		self.__dialog.set_snippet(snippet)
		
		if self.__dialog.run(ACTION_NEW) == RESPONSE_OK:
			snippet = self.__dialog.get_snippet()
			self.__settings.save_snippet(snippet)
			self.__update_snippets()
		
		self.__dialog.hide()
	
	def __update_snippets(self):
		self.__snippets = self.__settings.list_snippets()
		self.__view.set_snippets(self.__snippets)
	
	def __drag_text_received(self, view, text):
		self.__add_snippet(text)
	
	def __snippet_activated(self, view, index):
		helper = self.__window_helper.get_document_helper_manager().get_active_helper()
		if helper.get_language() == LANGUAGE_LATEX:
			source = self.__snippets[index].source
			helper.get_language_helper().insert_source(source)
	
	def __remove_snippet(self, view, index):
		snippet = self.__snippets[index]
		self.__settings.delete_snippet(snippet)
		self.__update_snippets()
	
	def __edit_snippet(self, view, index):
		if not self.__dialog:
			self.__dialog = SnippetDialog()
		
		self.__dialog.set_snippet(self.__snippets[index])
		
		if self.__dialog.run(ACTION_EDIT) == RESPONSE_OK:
			snippet = self.__dialog.get_snippet()
			self.__settings.update_snippet(snippet)
			self.__update_snippets()
		
		self.__dialog.hide()
		